searchNodes=[{"doc":"Type analysis system for Elixir.Mavis implements the Type module, which contains a type analysis system specifically tailored for the BEAM VM. The following considerations went into its design.Must be compatible with the existing dialyzer/typespec systemMay extend the typespec system if it's unobtrusive and can be, at a minimum, 'opt-out'.Does not have to conform to existing theoretical typesystems (e.g. H-M)Take maximum advantage of Elixir programming features to achieve readability and extensibility.Does not have to be easily usable from erlang, but must be able to handle modules produced in erlang.Compile-Time UsageThe type analysis system is designed to be a backend for a compile-time typechecker (see Selectrix). This system will infer the types of functions in modules and emit errors and warnings if there appear to be conflicts.One key function enabling this is fetch_type!/3; you can use this function to retrieve typing information on typing information stored in modules that have already been compiled.iex&gt; inspect Type.fetch_type!(String, :t, []) &quot;binary()&quot;Runtime UsageThere's no reason why you have to use the typing library exclusively at compile-time. Here is an example of using it at runtime:defmodule TestJson do @type json :: String.t | number | boolean | nil | [json] | %{optional(String.t) =&gt; json} def validate_json(data) do json_type = Type.fetch_type!(__MODULE__, :json, []) if Type.type_match?(json_type, data), do: :ok, else: raise &quot;not json&quot; end endNote that the above example is not particularly performant.Examples:iex&gt; import Type, only: :macros iex&gt; inspect Type.union(builtin(:non_neg_integer), :infinity) &quot;timeout()&quot; iex&gt; Type.intersection(builtin(:pos_integer), -10..10) 1..10The Type module implements two things:Core functionality for all typesytem operations.Support data structure for generic builtin and remote types.Type representation in MavisThe Type data structure is a struct with three parameters:module: The module in which the type is defined; nil for builtins.name: (atom) of the typeparams: a list of type arguments to the type data structure. These must be types themselves &quot;as applied&quot; to the type definition if we consider it to be a function.Representing other types.The following literals are represented directly:integersRanges (must be increasing)atomsempty list ([])The following types have associated structs:lists: Type.List.t/0bitstrings/binaries: Type.Bitstring.t/0tuples: Type.Tuple.t/0maps: Type.Map.t/0funs: Type.Function.t/0The following &quot;type containers&quot; have associated structs:unions: Type.Union.t/0opaque types: Type.Opaque.t/0function vars: Type.Function.Var.t/0Supported Type operationsThe Mavis typesystem provides five primary operations, which might not necessarily be the set of operations that one expects from a typesystem. These operations are chosen specifically reflect the needs of Erlang and Elixir's dynamic types and the type specification system provided by dialyzer.Type.union/1Type.intersection/1Type.subtype?/2Type.usable_as/3Type.of/1The operation Type.type_match?/2 is also provided, which is a combination of Type.of/1 and Type.subtype?/2.Deviations from standard Elixir and ErlangThe Curious case of String.tString.t/0 has a special meaning in Elixir, it is a UTF-8 encoded binary/0. As such, it is special-cased to have some properties that other remote types don't have out of the box. This sort of behaviour may be changed to be extensible to custom types in a future release.The nonexistent type String.t/1 is also implemented, with the type parameter indicating byte-lengths for compile-time constant strings. This is done entirely under the hood and should not otherwise affect operations. If you encounter strange results, report them to the issue tracker.In the meantime, you may disable this feature by setting the following:config :mavis, :use_smart_strings, false&quot;Aliased builtins&quot;Some builtins were not directly introduced into the typesystem logic, since they are easily represented as aliases or composite types. The following aliased builtins are usable with the builtin/1 macro, but will return false with is_builtin/1NB in the future the name of is_builtin may change to prevent confusion.term/0integer/0non_neg_integer/0arity/0as_boolean/1binary/0bitstring/0byte/0char/0charlist/0nonempty_charlist/0fun/0function/0identifier/0iodata/0keyword/0keyword/1list/0nonempty_list/0maybe_improper_list/0nonempty_maybe_improper_list/0mfa/0no_return/0number/0struct/0timeout/0iex&gt; import Type, only: :macros iex&gt; builtin(:timeout) %Type.Union{of: [:infinity, %Type{name: :pos_integer}, 0]} iex&gt; Type.is_builtin(builtin(:timeout)) falseModule and Node detailThe module/0 and node/0 types are given extra protection.An atom will not be considered a module unless it is detected to exist in the VM; although for usable_as/3 it will return the :maybe result if unconfirmed.iex&gt; import Type, only: :macros iex&gt; Type.type_match?(builtin(:module), :foo) false iex&gt; Type.type_match?(builtin(:module), Kernel) true iex&gt; Type.type_match?(builtin(:module), :gen_server) true iex&gt; Type.usable_as(Enum, builtin(:module)) :ok iex&gt; Type.usable_as(:not_a_module, builtin(:module)) {:maybe, [%Type.Message{target: %Type{name: :module}, type: :not_a_module}]}A node will not be considered a node unless it has the proper form for a node. usable_as/3 does not check active node lists, however.iex&gt; import Type, only: :macros iex&gt; Type.type_match?(builtin(:node), :foo) false iex&gt; Type.type_match?(builtin(:node), :nonode@nohost) true","ref":"Type.html","title":"Type","type":"module"},{"doc":"helper macro to match on builtin types. The parameter must be known at compile time. For the equivalent runtime macro, use: Type.select_builtin/1.Example:iex&gt; Type.builtin(:pos_integer) %Type{name: :pos_integer}Usable in guards","ref":"Type.html#builtin/1","title":"Type.builtin/1","type":"macro"},{"doc":"Types have an order that facilitates calculation of collapsing values into unions.Conforms to Elixir's compare api, so you can use this in Enum.sort/2For literals this follows the order in the erlang type system. Where one type is a strict subtype of another, it should wind up less than its supertypeTypes are organized into groups, which exist as a fastlane for comparing order between two different types (see typegroup/1).The order is as follows:group 0: none/0 and remote typesgroup 1 (integers):[negative integer literal]neg_integer/0[nonnegative integer literal]pos_integer/0group 2: float/0group 3 (atoms):[atom literal]node/0module/0atom/0group 4: reference/0group 5 (Type.Function.t/0):params: list functions (ordered by retval, then params in dictionary order)params: :any functions (ordered by retval, then params in dictionary order)group 6: port/0group 7: pid/0group 8 (Type.Tuple.t/0):defined tuples, in ascending order of arity, with cartesian dictionary ordering intrenally within an arity group.minimum size tuples, in descending order of size.group 9 (Type.Map.t/0): mapsgroup 10 (Type.List.t/0):nonempty: true listempty list literalnonempty: false listsgroup 11 (Type.Bitstring.t/0): bitstrings and binariesgroup 12: any/0Range.t/0 (group 1) comes after the highest integer in the range, with wider ranges coming after narrower ranges.iolist/0 (group 10) comes in the appropriate place in the list group.a member of Type.Union.t/0 comes after the highest represented item in its union.Examplesiex&gt; import Type, only: :macros iex&gt; Type.compare(builtin(:integer), builtin(:pid)) :lt iex&gt; Type.compare(-5..5, 1..5) :gt","ref":"Type.html#compare/2","title":"Type.compare/2","type":"function"},{"doc":"true if the list of types constitutes a complete cover of the provided type.see https://en.wikipedia.org/wiki/Cover_(topology)iex&gt; Type.covered?(-5..5, [1..5, 0, -5..-1]) true iex&gt; Type.covered?(-5..5, [1..5, 0, -5..-2]) false","ref":"Type.html#covered?/2","title":"Type.covered?/2","type":"function"},{"doc":"retrieves a typespec for a function, and converts it to a Type.t/0 value.Example:iex&gt; {:ok, spec} = Type.fetch_spec(String, :split, 1) iex&gt; inspect spec &quot;(String.t() -&gt; list(String.t()))&quot;","ref":"Type.html#fetch_spec/3","title":"Type.fetch_spec/3","type":"function"},{"doc":"retrieves a stored type from a module, and converts it to a Type.t/0 value.Example:iex&gt; {:ok, type} = Type.fetch_type(String, :t) iex&gt; inspect type &quot;binary()&quot;If the type has non-zero arity, you can specify its passed parameters as the third argument.","ref":"Type.html#fetch_type/4","title":"Type.fetch_type/4","type":"function"},{"doc":"resolves a remote type into its constitutent type. raises if the type is not found.","ref":"Type.html#fetch_type!/1","title":"Type.fetch_type!/1","type":"function"},{"doc":"see Type.fetch_type/4, except raises if the type is not found.","ref":"Type.html#fetch_type!/4","title":"Type.fetch_type!/4","type":"function"},{"doc":"generates the function type from a type-like ast. Note that the AST must be in a parentheses set. If the parameters are ..., it will generate the any function; for the generic n-arity function use _ for each of the parameters.Examples:iex&gt; import Type, only: :macros iex&gt; function (builtin(:atom) -&gt; builtin(:pos_integer)) %Type.Function{params: [%Type{name: :atom}], return: %Type{name: :pos_integer}} iex&gt; function (... -&gt; builtin(:pos_integer)) %Type.Function{params: :any, return: %Type{name: :pos_integer}} iex&gt; function (_, _ -&gt; builtin(:pos_integer)) %Type.Function{params: 2, return: %Type{name: :pos_integer}}If you want to tag function variables or constrain them, you can pass a keyword or atom list to the second parameter. These variables must appear in the return.iex&gt; import Type, only: :macros iex&gt; function (i -&gt; i when i: var) %Type.Function{params: [%Type.Function.Var{name: :i}], return: %Type.Function.Var{name: :i}} iex&gt; function (i -&gt; i when i: builtin(:pos_integer)) %Type.Function{params: [%Type.Function.Var{name: :i, constraint: %Type{name: :pos_integer}}], return: %Type.Function.Var{name: :i, constraint: %Type{name: :pos_integer}}}","ref":"Type.html#function/1","title":"Type.function/1","type":"macro"},{"doc":"outputs the type which is guaranteed to satisfy the following conditions:if a term is in all of the types in the list, it is in the result type.if a term is not in any of the types in the list, it is not in the result type.Example:iex&gt; import Type, only: :macros iex&gt; Type.intersection([builtin(:pos_integer), -1..10, -6..6]) 1..6","ref":"Type.html#intersection/1","title":"Type.intersection/1","type":"function"},{"doc":"outputs the type which is guaranteed to satisfy the following conditions:if a term is in both types, it is in the result type.if a term is not in either type, it is not in the result type.Example:iex&gt; import Type, only: :macros iex&gt; Type.intersection(builtin(:non_neg_integer), -10..10) 0..10","ref":"Type.html#intersection/2","title":"Type.intersection/2","type":"function"},{"doc":"guard that tests if the selected type is builtinExample:iex&gt; Type.is_builtin(:foo) false iex&gt; Type.is_builtin(%Type{name: :integer}) true iex&gt; Type.is_builtin(%Type{module: String, name: :t}) falseNote that composite builtin types may not match with this function:iex&gt; import Type, only: :macros iex&gt; Type.is_builtin(builtin(:mfa)) false","ref":"Type.html#is_builtin/1","title":"Type.is_builtin/1","type":"macro"},{"doc":"guard that tests if the selected type is remoteExample:iex&gt; Type.is_remote(:foo) false iex&gt; Type.is_remote(%Type{name: :integer}) false iex&gt; Type.is_remote(%Type{module: String, name: :t}) true","ref":"Type.html#is_remote/1","title":"Type.is_remote/1","type":"macro"},{"doc":"guard that tests if the selected type is a singleton type. This is a type that has only one value associated with it.Example:iex&gt; Type.is_singleton(:foo) true iex&gt; Type.is_singleton(%Type{name: :any}) false","ref":"Type.html#is_singleton/1","title":"Type.is_singleton/1","type":"macro"},{"doc":"generates a list of a particular type. A last parameter of ... indicates that the list should be nonemptyExamples:iex&gt; import Type, only: :macros iex&gt; list(...) %Type.List{type: %Type{name: :any}, nonempty: true} iex&gt; list(1..10) %Type.List{type: 1..10} iex&gt; list(1..10, ...) %Type.List{type: 1..10, nonempty: true}if it's passed a keyword list, it is interpreted as a keyword list.iex&gt; import Type, only: :macros iex&gt; list(foo: builtin(:pos_integer)) %Type.List{type: %Type.Tuple{elements: [:foo, %Type{name: :pos_integer}]}}","ref":"Type.html#list/1","title":"Type.list/1","type":"macro"},{"doc":"","ref":"Type.html#list/2","title":"Type.list/2","type":"macro"},{"doc":"generates the map type from a map ast. Unspecified keys default to required if singletons, and optional if non-singletons.iex&gt; import Type, only: :macros iex&gt; map %{foo: builtin(:pos_integer)} %Type.Map{required: %{foo: %Type{name: :pos_integer}}} iex&gt; map %{required(1) =&gt; builtin(:atom)} %Type.Map{required: %{1 =&gt; %Type{name: :atom}}} iex&gt; map %{optional(:bar) =&gt; builtin(:atom)} %Type.Map{optional: %{bar: %Type{name: :atom}}}","ref":"Type.html#map/1","title":"Type.map/1","type":"macro"},{"doc":"returns the type of the term.Examples:iex&gt; Type.of(47) 47 iex&gt; inspect Type.of(47.0) &quot;float()&quot; iex&gt; inspect Type.of([:foo, :bar]) &quot;list(:bar | :foo, ...)&quot; iex&gt; inspect Type.of([:foo | :bar]) &quot;nonempty_improper_list(:foo, :bar)&quot;Note that for functions, this may not be correct unless you supply an inference engine (see Type.Function):iex&gt; inspect Type.of(&amp;(&amp;1 + 1)) &quot;(any() -&gt; any())&quot;For maps, atom and number literals are marshalled into required terms; other literals, like strings, are marshalled into optional terms.iex&gt; inspect Type.of(%{foo: :bar}) &quot;map(%{foo: :bar})&quot; iex&gt; inspect Type.of(%{1 =&gt; :one}) &quot;map(%{1 =&gt; :one})&quot; iex&gt; inspect Type.of(%{&quot;foo&quot; =&gt; :bar, &quot;baz&quot; =&gt; &quot;quux&quot;}) &quot;map(%{optional(String.t()) =&gt; :bar | String.t()})&quot; iex&gt; inspect Type.of(1..10) &quot;map(%Range{first: 1, last: 10})&quot;","ref":"Type.html#of/1","title":"Type.of/1","type":"function"},{"doc":"partitions a type across a list of typessee https://en.wikipedia.org/wiki/Partition_of_a_setnote however, that if some part of your type is not represented in the type list that is provided, those members will be discarded.iex&gt; import Type, only: :macros iex&gt; Type.partition(-5..5, builtin(:integer).of) [1..5, 0, -5..-1]","ref":"Type.html#partition/2","title":"Type.partition/2","type":"function"},{"doc":"helper macro to match on remote typesExample:iex&gt; Type.remote(String.t()) %Type{module: String, name: :t}","ref":"Type.html#remote/1","title":"Type.remote/1","type":"macro"},{"doc":"use this for when you must use a runtime value to obtain a builtin type structnot usable in guards","ref":"Type.html#select_builtin/1","title":"Type.select_builtin/1","type":"macro"},{"doc":"outputs whether one type is a subtype of itself. To be true, the following condition must be satisfied:if a term is in the first type, then it is also in the second type.Note that any type is automatically a subtype of itself.Examples:iex&gt; import Type, only: :macros iex&gt; Type.subtype?(10, 1..47) true iex&gt; Type.subtype?(10, builtin(:integer)) true iex&gt; Type.subtype?(1..47, builtin(:integer)) true iex&gt; Type.subtype?(builtin(:integer), 1..47) false iex&gt; Type.subtype?(1..47, 1..47) trueRemote TypesRemote types are considered to be a signal that terms in the remote type satisfy &quot;special properties&quot;. For example, String.t/0 terms are not only binaries, but are UTF-8 encoded binaries. Thus a remote type is considered to be the subtype of its specification, but not vice versa:iex&gt; import Type, only: :macros iex&gt; binary = %Type.Bitstring{size: 0, unit: 8} iex&gt; Type.subtype?(remote(String.t()), binary) true iex&gt; Type.subtype?(binary, remote(String.t())) false","ref":"Type.html#subtype?/2","title":"Type.subtype?/2","type":"function"},{"doc":"generates the tuple type from a tuple ast. If the tuple contains ... it will generate the generic any tuple.See Type.Tuple for an explanation of deviations from dialyzer in the implementation of this type.iex&gt; import Type, only: :macros iex&gt; tuple {...} %Type.Tuple{elements: {:min, 0}} iex&gt; tuple {:ok, builtin(:pos_integer)} %Type.Tuple{elements: [:ok, %Type{name: :pos_integer}]} iex&gt; tuple {:error, builtin(:atom), builtin(:pos_integer)} %Type.Tuple{elements: [:error, %Type{name: :atom}, %Type{name: :pos_integer}]}","ref":"Type.html#tuple/1","title":"Type.tuple/1","type":"macro"},{"doc":"true if the passed term is an element of the type.Important:Note the argument order for this function, it does not have the same call order, as say, JavaScript's instanceof, or ruby's .is_a?Example:iex&gt; import Type, only: :macros iex&gt; Type.type_match?(builtin(:integer), 10) true iex&gt; Type.type_match?(builtin(:neg_integer), 10) false iex&gt; Type.type_match?(builtin(:pos_integer), 10) true iex&gt; Type.type_match?(1..9, 10) false iex&gt; Type.type_match?(-47..47, 10) true iex&gt; Type.type_match?(42, 10) false iex&gt; Type.type_match?(10, 10) true","ref":"Type.html#type_match?/2","title":"Type.type_match?/2","type":"function"},{"doc":"The typegroup of the type.This is a 'fastlane' value which simplifies generating type ordering code. See Type.compare/2 for a list of which groups the types belong to.NB: group assignments may change.","ref":"Type.html#typegroup/1","title":"Type.typegroup/1","type":"function"},{"doc":"outputs the type which is guaranteed to satisfy the following conditions:if a term is in any of the types, it is in the result type.if a term is not in any type, it is not in the result type.union/1 will try to collapse types into the simplest representation, but the success of this operation is not guaranteed.Example:iex&gt; import Type, only: :macros iex&gt; inspect Type.union([builtin(:pos_integer), -10..10, 32, builtin(:neg_integer)]) &quot;integer()&quot;","ref":"Type.html#union/1","title":"Type.union/1","type":"function"},{"doc":"outputs the type which is guaranteed to satisfy the following conditions:if a term is in either type, it is in the result type.if a term is not in either type, it is not in the result type.union/2 will try to collapse types into the simplest representation, but the success of this operation is not guaranteed.Example:iex&gt; import Type, only: :macros iex&gt; inspect Type.union(builtin(:pos_integer), -10..10) &quot;-10..-1 | non_neg_integer()&quot;","ref":"Type.html#union/2","title":"Type.union/2","type":"function"},{"doc":"Main utility function for determining type correctness.Answers the question: If a system claims to require a certain &quot;target type&quot; to execute without crashing, what will happen if send a term that satisfies a &quot;challenge type&quot;?The result may be one of::ok, which signals that no crash will occur{:maybe, [messages]}, which signals that a crash may occur due to one of the listed potential problems, but there are terms which will not trigger a crash.{:error, message} which signals that all terms in the challenge type will trigger a crash.These three levels are intended to roughly correspond to:&quot;no notification to the user&quot;&quot;emit a warning using IO.warn/2&quot;&quot;halt compilation with CompileError&quot;for a running compile-time analysis.usable_as/3 also may be passed metadata which can be used to correctly craft warning and error messages; as well as being filters for user-defined exceptions to warning or error rules.Relationship to subtype/2at first glance, it would seem that the subtype?/2 function is equivalent to usable_as/3, but there are cases where the relationship is not as clear. For example, if a function has the signature:(integer() -&gt; integer()), that is not necessarily usable as a function that is (any() -&gt; integer()), since it may be sent a value outside of the integers. Conversely an (any() -&gt; integer()) function IS usable as an (integer() -&gt; integer()) function. The subtyping relationship between these function types is unclear; in the Mavis system they are considered to be independent functions that are not subtypes of each other.Examples:iex&gt; import Type, only: :macros iex&gt; Type.usable_as(1, builtin(:integer)) :ok iex&gt; Type.usable_as(1, builtin(:neg_integer)) {:error, %Type.Message{type: 1, target: builtin(:neg_integer)}} iex&gt; Type.usable_as(-10..10, builtin(:neg_integer)) {:maybe, [%Type.Message{type: -10..10, target: builtin(:neg_integer)}]}Remote types:A remote type is intended to indicate that there is a quality outside of the type system which specifies the type. Thus, a remote type should be usable as the type it encapsulates, but it should emit a maybe when going the other direction:iex&gt; import Type, only: :macros iex&gt; binary = %Type.Bitstring{size: 0, unit: 8} iex&gt; Type.usable_as(remote(String.t()), binary) :ok iex&gt; Type.usable_as(binary, remote(String.t)) {:maybe, [%Type.Message{ type: binary, target: remote(String.t()), meta: [message: &quot;&quot;&quot; binary() is an equivalent type to String.t() but it may fail because it is a remote encapsulation which may require qualifications outside the type system. &quot;&quot;&quot;]}]}","ref":"Type.html#usable_as/3","title":"Type.usable_as/3","type":"function"},{"doc":"No members of this type will succeed in the operation.should typically result in a compile-time error.output by Type.usable_as/3","ref":"Type.html#t:error/0","title":"Type.error/0","type":"type"},{"doc":"type of group assignments","ref":"Type.html#t:group/0","title":"Type.group/0","type":"type"},{"doc":"Represents that some but not all members of the type will succeed in the operation.should typically result in a compile-time warning.output by Type.usable_as/3","ref":"Type.html#t:maybe/0","title":"Type.maybe/0","type":"type"},{"doc":"","ref":"Type.html#t:t/0","title":"Type.t/0","type":"type"},{"doc":"output type for Type.usable_as/3","ref":"Type.html#t:ternary/0","title":"Type.ternary/0","type":"type"},{"doc":"","ref":"Type.Helpers.html","title":"Type.Helpers","type":"behaviour"},{"doc":"Wraps the Type.Helpers.group_compare/2 function headers in a common prologue. This prevents errors from being made in code that must be common to all types.Prologue function matches:matches identical types and reports equalitymatches String.t/0 special case (only for Type.Bitstring)compares union types and puts them incompares opaque typescompares function var types","ref":"Type.Helpers.html#group_compare/1","title":"Type.Helpers.group_compare/1","type":"macro"},{"doc":"Type modules should implement this function to perform comparisons against all other types in their type group.See Type.typegroup/1","ref":"Type.Helpers.html#c:group_compare/2","title":"Type.Helpers.group_compare/2","type":"callback"},{"doc":"Wraps the Type.Properties.intersection/2 function headers in a common prologue. This prevents errors from being made in code that must be common to all types.Prologue function matches:matches itself and returns itselfmatches any/0 and returns itselfcalculates intersections for remote typesintersects union typesintersects function var types","ref":"Type.Helpers.html#intersection/1","title":"Type.Helpers.intersection/1","type":"macro"},{"doc":"Wraps the Type.Properties.subtype?/2 function headers in a common prologue. This prevents errors from being made in code that must be common to all types.Prologue function matches:matches itself and returns truematches none/0 and returns falsematches any/0 and returns anycalculates intersections for remote typesintersects union typesintersects function var typescan also take :usable_as as an argument; this declares that subtype?/2 is equivalent to an :ok response from Type.usable_as/3.","ref":"Type.Helpers.html#subtype/1","title":"Type.Helpers.subtype/1","type":"macro"},{"doc":"Wraps the Type.Properties.usable_as/3 function headers in common prologue and fallback clauses. This prevents errors from being made in code that must be common to all types.Prologue function matches:matches equal types and makes them output :okmatches usable_as with builtin(:any) and makes them output :okFallback function matches:catches usable_as against unions; and performs the appropriate attempt to match into each of the union's subtypes.catches all other attempts to run usable_as, and returns {:error, metadata}","ref":"Type.Helpers.html#usable_as/1","title":"Type.Helpers.usable_as/1","type":"macro"},{"doc":"TBD","ref":"Type.Inference.Api.html","title":"Type.Inference.Api","type":"behaviour"},{"doc":"","ref":"Type.Inference.Api.html#c:infer/3","title":"Type.Inference.Api.infer/3","type":"callback"},{"doc":"composes inference modules into a single inference pipeline.","ref":"Type.Inference.Pipeline.html","title":"Type.Inference.Pipeline","type":"module"},{"doc":"creates a new inference modules out others. Order is important, subsequent modules should be considered to be fallbacks to previous ones. For example, you might want to put caches in the front, followed by user overrides, followed by type-spec derived inferences, followed by computed inference.usage:First, define the pipeline.defpipeline PipelineModule, [Inference1, Inference2, Inference3]then you may use it as expected:PipelineModule.infer(String, :trim, 1)","ref":"Type.Inference.Pipeline.html#defpipeline/2","title":"Type.Inference.Pipeline.defpipeline/2","type":"macro"},{"doc":"TBA","ref":"Type.Message.html","title":"Type.Message","type":"module"},{"doc":"","ref":"Type.Message.html#make/3","title":"Type.Message.make/3","type":"function"},{"doc":"","ref":"Type.Message.html#t:t/0","title":"Type.Message.t/0","type":"type"},{"doc":"A fallback inference module. Builds a type out of the arity of the function, claiming that it takes any/0 values for all of the parameters and returns any/0","ref":"Type.NoInference.html","title":"Type.NoInference","type":"module"},{"doc":"Function which punts on inferring from MFA definitions.iex&gt; Type.NoInference.infer(String, :split, 1) {:ok, %Type.Function{params: [%Type{name: :any}], return: %Type{name: :any}}}","ref":"Type.NoInference.html#infer/3","title":"Type.NoInference.infer/3","type":"function"},{"doc":"Convenience functions. Should only be used in testing.","ref":"Type.Operators.html","title":"Type.Operators","type":"module"},{"doc":"","ref":"Type.Operators.html#%3C/2","title":"Type.Operators.</2","type":"function"},{"doc":"","ref":"Type.Operators.html#%3C=/2","title":"Type.Operators.<=/2","type":"function"},{"doc":"Shortcut for Type.union/2","ref":"Type.Operators.html#%3C%7C%3E/2","title":"Type.Operators.<|>/2","type":"function"},{"doc":"Shortcut for Type.intersection/2","ref":"Type.Operators.html#%3C~%3E/2","title":"Type.Operators.<~>/2","type":"function"},{"doc":"","ref":"Type.Operators.html#%3E/2","title":"Type.Operators.>/2","type":"function"},{"doc":"Shortcut for Type.compare/2","ref":"Type.Operators.html#%3E=/2","title":"Type.Operators.>=/2","type":"function"},{"doc":"Shortcut for Type.usable_as/2","ref":"Type.Operators.html#~%3E/2","title":"Type.Operators.~>/2","type":"function"},{"doc":"shortcut for Type.subtype?/2","ref":"Type.Operators.html#in/2","title":"Type.Operators.in/2","type":"function"},{"doc":"","ref":"Type.Properties.html","title":"Type.Properties","type":"protocol"},{"doc":"","ref":"Type.Properties.html#compare/2","title":"Type.Properties.compare/2","type":"function"},{"doc":"","ref":"Type.Properties.html#intersection/2","title":"Type.Properties.intersection/2","type":"function"},{"doc":"","ref":"Type.Properties.html#subtype?/2","title":"Type.Properties.subtype?/2","type":"function"},{"doc":"","ref":"Type.Properties.html#typegroup/1","title":"Type.Properties.typegroup/1","type":"function"},{"doc":"","ref":"Type.Properties.html#usable_as/3","title":"Type.Properties.usable_as/3","type":"function"},{"doc":"","ref":"Type.Properties.html#t:t/0","title":"Type.Properties.t/0","type":"type"},{"doc":"An inference module which looks up the spec of the function and if it can find it, it reports that as its associated type.","ref":"Type.SpecInference.html","title":"Type.SpecInference","type":"module"},{"doc":"Exampleiex&gt; Type.SpecInference.infer(String, :trim, 1) %Type.Function{params: [%Type{module: String, name: :t}], return: %Type{module: String, name: :t}}","ref":"Type.SpecInference.html#infer/3","title":"Type.SpecInference.infer/3","type":"function"},{"doc":"NOT IMPLEMENTED YET.","ref":"Type.AsBoolean.html","title":"Type.AsBoolean","type":"module"},{"doc":"Handles bitstrings and binaries in the erlang/elixir type system.This type has two required parameters that define a semilattice over the number line which are the allowed number of bits in the bitstrings which are members of this type.size minimum size of the bitstring.unit distance between points in the lattice.Roughly speaking, this corresponds to the process of matching the header of a binary (size), plus a variable-length payload with recurring features of size (unit).Examples:bitstring/0 is equivalent to %Type.Bitstring{size: 0, unit: 1}. Note that any bitstring type is a subtype of this type.binary/0 is equivalent to %Type.Bitstring{size: 0, unit: 8}.A fixed-size binary is %Type.Bitstring{unit: 0, size: &lt;size&gt;}.The empty binary (&quot;&quot;) is %Type.Bitstring{unit: 0, size: 0}.Key functions:comparisonbinaries are sorted by unit first (with smaller units coming after bigger units, as they are less restrictive), except zero, which is the most restrictive.iex&gt; Type.compare(%Type.Bitstring{size: 0, unit: 8}, %Type.Bitstring{size: 0, unit: 1}) :lt iex&gt; Type.compare(%Type.Bitstring{size: 0, unit: 1}, %Type.Bitstring{size: 16, unit: 0}) :gtfor two binaries with the same unit, a binary with a smaller fixed size comes after those with bigger fixed sizes, since bigger fixed sizes are more restrictive.iex&gt; Type.compare(%Type.Bitstring{size: 0, unit: 8}, %Type.Bitstring{size: 16, unit: 8}) :gt iex&gt; Type.compare(%Type.Bitstring{size: 16, unit: 8}, %Type.Bitstring{size: 8, unit: 8}) :ltintersectionSome binaries have no intersection, but Mavis will find obscure intersections.iex&gt; Type.intersection(%Type.Bitstring{size: 15, unit: 0}, %Type.Bitstring{size: 3, unit: 0}) %Type{name: :none} iex&gt; Type.intersection(%Type.Bitstring{size: 0, unit: 8}, %Type.Bitstring{size: 16, unit: 4}) %Type.Bitstring{size: 16, unit: 8} iex&gt; Type.intersection(%Type.Bitstring{size: 15, unit: 1}, %Type.Bitstring{size: 0, unit: 8}) %Type.Bitstring{size: 16, unit: 8} iex&gt; Type.intersection(%Type.Bitstring{size: 14, unit: 6}, %Type.Bitstring{size: 16, unit: 8}) %Type.Bitstring{size: 32, unit: 24}unionMost binary pairs won't have nontrivial unions, but Mavis will find some obscure ones.iex&gt; Type.union(%Type.Bitstring{size: 0, unit: 1}, %Type.Bitstring{size: 15, unit: 8}) %Type.Bitstring{size: 0, unit: 1} iex&gt; Type.union(%Type.Bitstring{size: 25, unit: 8}, %Type.Bitstring{size: 1, unit: 4}) %Type.Bitstring{size: 1, unit: 4}subtype?A bitstring type is a subtype of another if its lattice is covered by the other's.iex&gt; Type.subtype?(%Type.Bitstring{size: 16, unit: 8}, %Type.Bitstring{size: 4, unit: 4}) true iex&gt; Type.subtype?(%Type.Bitstring{size: 3, unit: 7}, %Type.Bitstring{size: 12, unit: 6}) falseusable_asMost bitstrings are at least maybe usable as others, but there are cases where it's impossible.iex&gt; Type.usable_as(%Type.Bitstring{size: 8, unit: 16}, %Type.Bitstring{size: 4, unit: 4}) :ok iex&gt; Type.usable_as(%Type.Bitstring{size: 3, unit: 7}, %Type.Bitstring{size: 12, unit: 6}) {:maybe, [%Type.Message{meta: [], type: %Type.Bitstring{size: 3, unit: 7}, target: %Type.Bitstring{size: 12, unit: 6}}]} iex&gt; Type.usable_as(%Type.Bitstring{size: 3, unit: 8}, %Type.Bitstring{size: 16, unit: 8}) {:error, %Type.Message{meta: [], type: %Type.Bitstring{size: 3, unit: 8}, target: %Type.Bitstring{size: 16, unit: 8}}}","ref":"Type.Bitstring.html","title":"Type.Bitstring","type":"module"},{"doc":"","ref":"Type.Bitstring.html#t:t/0","title":"Type.Bitstring.t/0","type":"type"},{"doc":"Represents a function type.There are two fields for the struct defined by this module.params a list of types for the function arguments. Note that the arity of the function is the length of this list. May also be the atom :any which corresponds to &quot;a function of any arity&quot;.return the type of the returned value.Examples:(... -&gt; integer()) would be represented as %Type.Function{params: :any, return: %Type{name: :integer}}(integer() -&gt; integer()) would be represented as %Type.Function{params: [%Type{name: :integer}], return: %Type{name: :integer}}Shortcut FormThe Type module lets you specify a function using &quot;shortcut form&quot; via the Type.function/1 macro:iex&gt; import Type, only: :macros iex&gt; function((builtin(:atom) -&gt; builtin(:pos_integer))) %Type.Function{params: [%Type{name: :atom}], return: %Type{name: :pos_integer}}InferenceBy default, Mavis will not attempt to perform inference on function types.iex&gt; inspect Type.of(&amp;(&amp;1 + 1)) &quot;(any() -&gt; any())&quot;If you would like to perform inference on the function to obtain more details on the acceptable function types, set the inference environment variable. For example, if you're using the :mavis_inference hex package, do:Application.put_env(:mavis, :inference, Type.Inference)The default module for this is Type.NoInferenceKey functions:comparisonFunctions are ordered first by the type order on their return type, followed by type order on their parameters.iex&gt; import Type, only: :macros iex&gt; Type.compare(function(( -&gt; builtin(:atom))), function(( -&gt; builtin(:integer)))) :gt iex&gt; Type.compare(function((builtin(:integer) -&gt; builtin(:integer))), ...&gt; function((builtin(:atom) -&gt; builtin(:integer)))) :ltintersectionFunctions with distinct parameter types are nonoverlapping, even if their parameter types overlap. If they have the same parameters, then their return values are intersected.iex&gt; import Type, only: :macros iex&gt; Type.intersection(function(( -&gt; 1..10)), function(( -&gt; builtin(:integer)))) %Type.Function{params: [], return: 1..10} iex&gt; Type.intersection(function((builtin(:integer) -&gt; builtin(:integer))), ...&gt; function((1..10 -&gt; builtin(:integer)))) %Type{name: :none}functions with :any parameters intersected with a function with specified parameters will adopt the parameters of the intersected function.iex&gt; import Type, only: :macros iex&gt; Type.intersection(function((... -&gt; builtin(:pos_integer))), ...&gt; function((1..10 -&gt; builtin(:pos_integer)))) %Type.Function{params: [1..10], return: %Type{name: :pos_integer}}unionFunctions are generally not merged in union operations, but if their parameters are identical then their return types will be merged.iex&gt; import Type, only: :macros iex&gt; Type.union(function(( -&gt; 1..10)), function(( -&gt; 11..20))) %Type.Function{params: [], return: 1..20}subtype?A function type is the subtype of another if it has the same parameters and its return value type is the subtype of the other'siex&gt; import Type, only: :macros iex&gt; Type.subtype?(function((builtin(:integer) -&gt; 1..10)), ...&gt; function((builtin(:integer) -&gt; builtin(:integer)))) trueusable_asThe usable_as relationship for functions may not necessarily be obvious. An easy way to think about it, is: if I passed a function with this type to a function that demanded the other type how confident would I be that it would not crash.A function is usable_as another function if all of its parameters are supertypes of the targeted function; and if its return type is subtypes of the return type of the targeted function.iex&gt; import Type, only: :macros iex&gt; Type.usable_as(function((builtin(:pos_integer) -&gt; 1..10)), function((1..10 -&gt; builtin(:pos_integer)))) :ok iex&gt; Type.usable_as(function((1..10 -&gt; 1..10)), function((builtin(:pos_integer) -&gt; builtin(:pos_integer)))) {:maybe, [%Type.Message{type: %Type.Function{params: [1..10], return: 1..10}, target: %Type.Function{params: [%Type{name: :pos_integer}], return: %Type{name: :pos_integer}}}]} iex&gt; Type.usable_as(function(( -&gt; builtin(:atom))), function(( -&gt; builtin(:pos_integer)))) {:error, %Type.Message{type: %Type.Function{params: [], return: %Type{name: :atom}}, target: %Type.Function{params: [], return: %Type{name: :pos_integer}}}}","ref":"Type.Function.html","title":"Type.Function","type":"module"},{"doc":"applies types to a function definition.Raises with Type.FunctionError if one of the following is true:an :any function is attempted to be applieda top-arity function is attempted to be applieda non-function (or union of functions) is attempted to be appliedReturns{:ok, return_type} when the function call is successful.{:maybe, return_type, [messages]} when one or more of the parameters is overspecified.{:error, message} when any of the parameters is disjointExamples:iex&gt; import Type, only: :macros iex&gt; func = function ((builtin(:pos_integer) -&gt; builtin(:float))) iex&gt; Type.Function.apply_types(func, [builtin(:pos_integer)]) {:ok, %Type{name: :float}} iex&gt; Type.Function.apply_types(func, [builtin(:non_neg_integer)]) {:maybe, %Type{name: :float}, [ %Type.Message{ type: %Type.Union{of: [%Type{name: :pos_integer}, 0]}, target: %Type{name: :pos_integer}, meta: [message: &quot;non_neg_integer() is overbroad for argument 1 (pos_integer()) of function (pos_integer() -&gt; float())&quot;] }]} iex&gt; Type.Function.apply_types(func, [builtin(:float)]) {:error, %Type.Message{ type: %Type{name: :float}, target: %Type{name: :pos_integer}, meta: [message: &quot;float() is disjoint to argument 1 (pos_integer()) of function (pos_integer() -&gt; float())&quot;] }} iex&gt; var_func = function((i -&gt; i when i: builtin(:integer))) iex&gt; Type.Function.apply_types(var_func, [1..10]) {:ok, 1..10}","ref":"Type.Function.html#apply_types/3","title":"Type.Function.apply_types/3","type":"function"},{"doc":"","ref":"Type.Function.html#t:return/0","title":"Type.Function.return/0","type":"type"},{"doc":"","ref":"Type.Function.html#t:t/0","title":"Type.Function.t/0","type":"type"},{"doc":"a special container type indicating that the function has a type dependency.Example:The following typespec:@spec identity(x) :: x when x: vargenerates the following typespec:%Type.Function{ params: [%Type.Function.Var{name: :x}], return: %Type.Function.Var{name: :x} }if you further put a restriction on this typespec:@spec identity(x) :: x when x: integerthe Type.Function.Var will further exhibit the issued constraint:%Type.Function{ params: [%Type.Function.Var{name: :x, constraint: %Type{name: :integer}}], return: %Type.Function.Var{name: :x, constraint: %Type{name: :integer}} }","ref":"Type.Function.Var.html","title":"Type.Function.Var","type":"module"},{"doc":"","ref":"Type.Function.Var.html#t:t/0","title":"Type.Function.Var.t/0","type":"type"},{"doc":"Represents lists, except for the empty list, which is represented by the empty list literal [].There are three fields for the struct defined by this module.nonempty if true, the list must have at least one element; if false, then it may be the empty list [].type the type for all elements of the list, except the final elementfinal the type of the final element. Typically this is [], but other types may be used as the final element and these are called improper lists.Shortcut FormThe Type module lets you specify a list using &quot;shortcut form&quot; via the Type.list/1 and Type.list/2 macros:iex&gt; import Type, only: :macros iex&gt; list(builtin(:pos_integer)) %Type.List{type: %Type{name: :pos_integer}} iex&gt; list(...) %Type.List{type: %Type{name: :any}, nonempty: true} iex&gt; list(builtin(:pos_integer), ...) %Type.List{type: %Type{name: :pos_integer}, nonempty: true}Examples:The &quot;any proper list&quot; type is %Type.List{}. Note this is distinct from []iex&gt; inspect %Type.List{} &quot;list()&quot;a list of a given typeiex&gt; inspect %Type.List{type: %Type{name: :integer}} &quot;list(integer())&quot;a nonempty list of a given typeiex&gt; inspect %Type.List{type: %Type{name: :integer}, nonempty: true} &quot;list(integer(), ...)&quot;an improper list must be nonempty, and looks like the following:iex&gt; inspect %Type.List{type: %Type{module: String, name: :t}, ...&gt; nonempty: true, ...&gt; final: %Type{module: String, name: :t}} &quot;nonempty_improper_list(String.t(), String.t())&quot;a maybe improper list should have empty list as a subtype of the final field.iex&gt; inspect %Type.List{type: %Type{module: String, name: :t}, ...&gt; final: %Type.Union{of: [[], %Type{module: String, name: :t}]}} &quot;maybe_improper_list(String.t(), String.t())&quot;Key functions:comparisonnonempty: false list types come after nonempty: true list types; and list types are ordered by the type order of their content types, followed by the type order of their finals. The empty list type comes between the two nonempty categories.iex&gt; import Type, only: :macros iex&gt; Type.compare(list(...), []) :lt iex&gt; Type.compare(builtin(:list), []) :gt iex&gt; Type.compare(list(builtin(:integer)), list(builtin(:atom))) :lt iex&gt; Type.compare(%Type.List{final: builtin(:integer)}, %Type.List{final: builtin(:atom)}) :ltintersectionThe intersection of two list types is the intersection of their contents; a nonempty: true list type intersected with a nonempty: false list type is nonempty: trueiex&gt; import Type, only: :macros iex&gt; Type.intersection(list(...), builtin(:list)) %Type.List{nonempty: true} iex&gt; Type.intersection(list(1..20), list(10..30)) %Type.List{type: 10..20}unionThe intersection of two list types is the union of their contents; a nonempty: true list type intersected with a nonempty: false list type is nonempty: falseiex&gt; import Type, only: :macros iex&gt; Type.union(list(...), builtin(:list)) %Type.List{} iex&gt; Type.union(list(1..10), list(10..20)) %Type.List{type: 1..20}subtype?A list type is a subtype of another if its contents are subtypes of each other; a nonempty: true list type is subtype of its nonempty: false counterpart.iex&gt; import Type, only: :macros iex&gt; Type.subtype?(list(...), builtin(:list)) true iex&gt; Type.subtype?(list(1..10), list(2..30)) falseusable_asA list type is usable_as another if its contents are usable_as the other's; nonempty: false list types might be usable as nonempty: true types.iex&gt; import Type, only: :macros iex&gt; Type.usable_as(list(1..10), list(builtin(:integer))) :ok iex&gt; Type.usable_as(list(1..10), list(builtin(:atom))) # note it might be the empty list {:maybe, [%Type.Message{type: %Type.List{type: 1..10}, target: %Type.List{type: %Type{name: :atom}}}]} iex&gt; Type.usable_as(builtin(:list), list(...)) {:maybe, [%Type.Message{type: %Type.List{}, target: %Type.List{nonempty: true}}]}","ref":"Type.List.html","title":"Type.List","type":"module"},{"doc":"","ref":"Type.List.html#t:t/0","title":"Type.List.t/0","type":"type"},{"doc":"Represents map terms. Note that some of the choices around how to handle maps may deviate from the expectations in the typesystem.The associated struct has two parameters::required a map of required key types and their associated value types.:optional a map of optional key types and their associated value types.Shortcut FormThe Type module lets you specify a map using &quot;shortcut form&quot; via the Type.map/1 macro:Note that the empty map is not the same as map/0iex&gt; import Type, only: :macros iex&gt; map(%{optional(builtin(:atom)) =&gt; builtin(:pos_integer)}) %Type.Map{optional: %{%Type{name: :atom} =&gt; %Type{name: :pos_integer}}} iex&gt; map(%{}) # empty map %Type.Map{optional: %{}, required: %{}} iex&gt; builtin(:map) # t:map/0 %Type.Map{optional: %{%Type{name: :any} =&gt; %Type{name: :any}}}Deviations from standard Erlang/Elixir:Type.Map only allows literal integers and literal atoms as required key types. Typespecs that put other types in as required will downgrade those types to be optionalIn the future, this may be extended to ranges.map types will not by default assume optional(any()) =&gt; any()If multiple specifications exist for the same particular value (for example, overlapping ranges in optional keys, or a overlapping types in required vs optional), all specifications are applied to that value group. This may result in the map being equivalent to %Type{name: :none} but that will not automatically be collapsed to that value.Examples:The empty map is %Type.Map{}. This is not allowed to have any k/v pairs.iex&gt; inspect %Type.Map{} &quot;map(%{})&quot;A map with a required atom key might look as follows:iex&gt; inspect %Type.Map{required: %{foo: %Type{name: :integer}}} &quot;map(%{foo: integer()})&quot;A map with a required integer key might look as follows:iex&gt; inspect %Type.Map{required: %{1 =&gt; %Type{name: :integer}}}&quot;map(%{1 =&gt; integer()})&quot;A map with an optional key type might look as follows:iex&gt; inspect %Type.Map{optional: %{%Type{name: :integer} =&gt; %Type{name: :integer}}} &quot;map(%{optional(integer()) =&gt; integer()})&quot;The &quot;any&quot; map has optional any mapping to any (note this is distinct from empty map %{})iex&gt; inspect %Type.Map{optional: %{%Type{name: :any} =&gt; %Type{name: :any}}} &quot;map()&quot;Key functions:comparisonMaps are ordered by required keys, then optional keys. A map type with a required key comes before an equivalent map type with the key optional.iex&gt; import Type, only: :macros iex&gt; Type.compare(map(%{foo: :bar}), map(%{foo: :quux})) :lt iex&gt; Type.compare(map(%{foo: :bar}), map(%{bar: :baz})) :gt iex&gt; Type.compare(map(%{optional(1) =&gt; builtin(:integer), foo: builtin(:integer)}), ...&gt; map(%{optional(2) =&gt; builtin(:integer), foo: builtin(:integer)})) :lt iex&gt; Type.compare(map(%{foo: :bar}), map(%{optional(:foo) =&gt; :bar})) :ltintersectionIf map types cannot support the same required keys, their intersection is none() If one of the required keys can be created from the pool of the other key's optional types, Then it gets realized into the intersection. Otherwise, the intersection is the intersection of key types with key values. If the optional key or value types are totally disjoint, then the intersection is the empty map %{} because that term is a member of both typesiex&gt; import Type, only: :macros iex&gt; Type.intersection(map(%{foo: :bar}), map(%{bar: :baz})) %Type{name: :none} iex&gt; Type.intersection(map(%{foo: :bar}), map(%{optional(builtin(:atom)) =&gt; builtin(:atom)})) %Type.Map{required: %{foo: :bar}} iex&gt; Type.intersection(map(%{1 =&gt; 1..10}), map(%{1 =&gt; 5..20})) %Type.Map{required: %{1 =&gt; 5..10}} iex&gt; Type.intersection(map(%{optional(1..10) =&gt; builtin(:integer)}), ...&gt; map(%{optional(builtin(:integer)) =&gt; 1..10})) %Type.Map{optional: %{1..10 =&gt; 1..10}} iex&gt; Type.intersection(map(%{optional(1..10) =&gt; builtin(:integer)}), ...&gt; map(%{optional(1..10) =&gt; builtin(:atom)})) %Type.Map{} iex&gt; Type.intersection(map(%{optional(1..10) =&gt; builtin(:integer)}), ...&gt; map(%{optional(11..20) =&gt; builtin(:integer)})) %Type.Map{}unionMap unions must be extremely parsimonious, because the specifications on the unions are coupled. Not all combinations of optional and required types can be safely subjected to a union. In general, the union algorithm will merge two types if one is a strict subtype of the other.iex&gt; import Type, only: :macros iex&gt; Type.union(map(%{foo: :bar}), map(%{})) %Type.Map{optional: %{foo: :bar}} iex&gt; Type.union(map(%{foo: :bar}), map(%{optional(:foo) =&gt; :bar})) %Type.Map{optional: %{foo: :bar}} iex&gt; Type.union(map(%{foo: 1..10}), map(%{foo: 1..20})) %Type.Map{required: %{foo: 1..20}} iex&gt; Type.union(map(%{optional(1..10) =&gt; 1..10}), map(%{optional(1..20) =&gt; 1..10})) %Type.Map{optional: %{1..20 =&gt; 1..10}}subtype?A map type is a subtype of the other if all of the keys and values of one are subtypes of the other.iex&gt; import Type, only: :macros iex&gt; Type.subtype?(map(%{foo: :bar}), map(%{foo: builtin(:atom)})) true iex&gt; Type.subtype?(map(%{foo: :bar}), map(%{optional(:foo) =&gt; builtin(:atom)})) true iex&gt; Type.subtype?(map(%{optional(:foo) =&gt; :bar}), map(%{optional(:foo) =&gt; builtin(:atom)})) true iex&gt; Type.subtype?(map(%{optional(:foo) =&gt; :bar}), map(%{foo: builtin(:atom)})) falseusable_asoptional keys are maybe usable as required keys; required keys are always usable as optional keys.for optional keys, if a key type is a subtype of the target's key type, then it is maybe usable; if it as a supertype of the target's key type, then it is always usable.generally optional types are :ok with disjoint key types, but if overlapping key types have conflicting value types, then it is :maybe because keys must beiex&gt; import Type, only: :macros iex&gt; Type.usable_as(map(%{foo: :bar}), map(%{optional(:foo) =&gt; :bar})) :ok iex&gt; Type.usable_as(map(%{optional(:foo) =&gt; :bar}), map(%{foo: :bar})) {:maybe, [%Type.Message{type: %Type.Map{optional: %{foo: :bar}}, target: %Type.Map{required: %{foo: :bar}}}]} iex&gt; Type.usable_as(map(%{optional(1..10) =&gt; 1..10}), map(%{optional(1..20) =&gt; 11..20})) {:maybe, [%Type.Message{type: %Type.Map{optional: %{1..10 =&gt; 1..10}}, target: %Type.Map{optional: %{1..20 =&gt; 11..20}}}]}Helper functionsThe docs for helper functions documented here use set theory language, treating maps as discrete functions with preimages and images. This page may be of help to understand this language:https://en.wikipedia.org/wiki/Image_(mathematics)#Inverse_image","ref":"Type.Map.html","title":"Type.Map","type":"module"},{"doc":"calculates the image of a map when given a clamping type for the preimage.If any part of the clamp doesn't exist in in the map's preimage, it is ignored.Example:(note that 0 is in the clamp -5..5, but is not in the map preimage)iex&gt; alias Type.Map iex&gt; import Type, only: :macros iex&gt; Map.apply(map(%{builtin(:neg_integer) =&gt; :foo, ...&gt; builtin(:pos_integer) =&gt; :bar}), -5..5) %Type.Union{of: [:foo, :bar]}If any part of the clamp has conflicting definitions, it is dropped.iex&gt; alias Type.Map iex&gt; import Type, only: :macros iex&gt; Map.apply(map(%{0..3 =&gt; :foo, 4..5 =&gt; :bar, 4 =&gt; :baz, 5 =&gt; :quux}), 0..5) :fooIf any part of the clamp has a more restrictive definition, all type restrictions must applyiex&gt; alias Type.Map iex&gt; import Type, only: :macros iex&gt; Map.apply(map(%{0..3 =&gt; 1..10, builtin(:pos_integer) =&gt; 0..5}), 1..3) 1..5","ref":"Type.Map.html#apply/2","title":"Type.Map.apply/2","type":"function"},{"doc":"takes all required terms and makes them optional","ref":"Type.Map.html#optionalize/2","title":"Type.Map.optionalize/2","type":"function"},{"doc":"the full union of all possible key values for the passed map.iex&gt; alias Type.Map iex&gt; import Type, only: :macros iex&gt; Map.preimage(map(%{builtin(:pos_integer) =&gt; builtin(:any)})) %Type{name: :pos_integer} iex&gt; Map.preimage(map(%{0 =&gt; builtin(:any), builtin(:pos_integer) =&gt; builtin(:any)})) %Type.Union{of: [%Type{name: :pos_integer}, 0]}","ref":"Type.Map.html#preimage/1","title":"Type.Map.preimage/1","type":"function"},{"doc":"true if the map type specifies a singleton type as one of its required keys","ref":"Type.Map.html#required_key?/2","title":"Type.Map.required_key?/2","type":"function"},{"doc":"Takes a list of (disjoint) preimage types &quot;segments&quot; and splits them further in such a way that each segment has a consistent single type in the map's image.Discards any preimage subsegments which are not present in the map.Does not validate that passed list of preimages are actually disjoint.No guarantees are made on the order of the resulting list.iex&gt; alias Type.Map iex&gt; import Type, only: :macros iex&gt; Map.resegment(map(%{builtin(:neg_integer) =&gt; :neg}), [-10..10]) [-10..-1] iex&gt; Map.resegment(map(%{builtin(:neg_integer) =&gt; :neg, ...&gt; builtin(:pos_integer) =&gt; :pos}), [-10..10]) [-10..-1, 1..10]","ref":"Type.Map.html#resegment/2","title":"Type.Map.resegment/2","type":"function"},{"doc":"","ref":"Type.Map.html#t:optional/0","title":"Type.Map.optional/0","type":"type"},{"doc":"","ref":"Type.Map.html#t:required/0","title":"Type.Map.required/0","type":"type"},{"doc":"","ref":"Type.Map.html#t:t/0","title":"Type.Map.t/0","type":"type"},{"doc":"A wrapper for opaqueness.","ref":"Type.Opaque.html","title":"Type.Opaque","type":"module"},{"doc":"","ref":"Type.Opaque.html#t:t/0","title":"Type.Opaque.t/0","type":"type"},{"doc":"Represents tuple types.The associated struct has one parameter::elements which may be a list of types, corresponding to the ordered list of tuple element types.Deviations from standard Erlang/Elixir:Mavis introduces a new type (that is not expressible via dialyzer). This is a tuple with minimum arity (n), this is represented by putting the tuple {:min, n} in the :elements field. The properties of this type should be self-explanatory. The {:min, 0} tuple is equvalent to the standard Erlang type {...}, aka any tupleExamples:the any tuple is %Type.Tuple{elements: {:min, 0}}iex&gt; inspect %Type.Tuple{elements: {:min, 0}} &quot;tuple()&quot;A tuple of minimum size is represented as follows:iex&gt; inspect %Type.Tuple{elements: {:min, 2}} &quot;tuple({...(min: 2)})&quot;generic tuples have their types as lists.iex&gt; inspect %Type.Tuple{elements: [%Type{name: :atom}, %Type{name: :integer}]} &quot;tuple({atom(), integer()})&quot; iex&gt; inspect %Type.Tuple{elements: [:ok, %Type{name: :integer}]} &quot;tuple({:ok, integer()})&quot;Shortcut FormThe Type module lets you specify a tuple using &quot;shortcut form&quot; via the Type.tuple/1 macro:iex&gt; import Type, only: :macros iex&gt; tuple {...} %Type.Tuple{elements: {:min, 0}} iex&gt; tuple {...(min: 2)} %Type.Tuple{elements: {:min, 2}} iex&gt; tuple {:ok, builtin(:integer)} %Type.Tuple{elements: [:ok, builtin(:integer)]}Key functions:comparisonLonger tuples come after shorter tuples; tuples are then ordered using Cartesian dictionary order along the elements list.iex&gt; import Type, only: :macros iex&gt; Type.compare(tuple({}), tuple({:foo})) :lt iex&gt; Type.compare(tuple({:foo, 1..10}), tuple({:bar, 10..20})) :gtintersectionTuples of different length do not intersect; the intersection is otherwise the Cartesian intersection of the elements.iex&gt; import Type, only: :macros iex&gt; Type.intersection(tuple({}), tuple({:ok, builtin(:integer)})) %Type{name: :none} iex&gt; Type.intersection(tuple({:ok, builtin(:integer)}), tuple({builtin(:atom), 1..10})) %Type.Tuple{elements: [:ok, 1..10]}unionOnly tuple types of the same length can be non-trivially unioned, and then, only if one tuple type is a subtype of the other, and they must be identical across all but one dimension.iex&gt; import Type, only: :macros iex&gt; Type.union(tuple({:ok, 11..20}), tuple({:ok, 1..10})) %Type.Tuple{elements: [:ok, 1..20]}subtype?A tuple type is the subtype of another if its types are subtypes of the other across all Cartesian dimensions, but is not the subtype of a tuple that requires more minimum values.iex&gt; import Type, only: :macros iex&gt; Type.subtype?(tuple({:ok, 1..10}), tuple({builtin(:atom), builtin(:integer)})) true iex&gt; Type.subtype?(tuple({:ok, 1..10}), tuple({...(min: 2)})) true iex&gt; Type.subtype?(tuple({:ok, 1..10}), tuple({...(min: 3)})) falseusable_asA tuple type is usable as another if it each of its elements are usable as the other across all Cartesian dimensions. If any element is disjoint, then it is not usable.iex&gt; import Type, only: :macros iex&gt; Type.usable_as(tuple({:ok, 1..10}), tuple({builtin(:atom), builtin(:integer)})) :ok iex&gt; Type.usable_as(tuple({:ok, builtin(:integer)}), tuple({builtin(:atom), 1..10})) {:maybe, [%Type.Message{type: tuple({:ok, builtin(:integer)}), target: tuple({builtin(:atom), 1..10})}]} iex&gt; Type.usable_as(tuple({:ok, builtin(:integer)}), tuple({:error, 1..10})) {:error, %Type.Message{type: tuple({:ok, builtin(:integer)}), target: tuple({:error, 1..10})}}","ref":"Type.Tuple.html","title":"Type.Tuple","type":"module"},{"doc":"returns the tuple type at the (0-indexed) tuple slot.iex&gt; import Type, only: :macros iex&gt; Type.Tuple.elem(tuple({:ok, builtin(:pos_integer)}), 1) %Type{name: :pos_integer}","ref":"Type.Tuple.html#elem/2","title":"Type.Tuple.elem/2","type":"function"},{"doc":"a utility function which takes two type lists and ascertains if they can be merged as tuples.returns the merged list if the two lists are mergeable. This follows from one of two conditions:each type in the &quot;narrower&quot; type list is a subtype of the corresponding &quot;broader&quot; type listall types are equal except for onereturns nil if the two lists are not mergeable; returns the merged list if the two lists are mergeable.completes full analysis in a single pass of the function.Used in common between tuples and function parameters.","ref":"Type.Tuple.html#merge/2","title":"Type.Tuple.merge/2","type":"function"},{"doc":"","ref":"Type.Tuple.html#t:t/0","title":"Type.Tuple.t/0","type":"type"},{"doc":"Represents the Union of two or more types.The associated struct has one field::of which is a list of all types that are being unioned.For performance purposes, Union keeps its subtypes in reverse-type-order.Type.Union implements both the enumerable protocol and the collectable protocol; if you collect into a union, it will return a Type.t/0 result in general; it might not be a Type.Union struct:iex&gt; Enum.into([1, 3], %Type.Union{}) %Type.Union{of: [3, 1]} iex&gt; inspect %Type.Union{of: [3, 1]} &quot;1 | 3&quot; iex&gt; Enum.into([1..10, 11..20], %Type.Union{}) 1..20","ref":"Type.Union.html","title":"Type.Union","type":"module"},{"doc":"","ref":"Type.Union.html#t:t/0","title":"Type.Union.t/0","type":"type"},{"doc":"","ref":"Type.Union.html#t:t/1","title":"Type.Union.t/1","type":"type"},{"doc":"","ref":"Type.FunctionError.html","title":"Type.FunctionError","type":"exception"},{"doc":"Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"deprecate Type.Union.of/2, use Type.union/2 insteadchange builtin(:non_neg_integer) from a true builtin to an aliased builtin.change builtin(:integer) from a true builtin to an aliased builtinadds Type.map/1, Type.tuple/1, and Type.function/1 macrosadds Type.is_singleton/1 guardfixes tuple union merging rules and makes function union merging rulesadds top-arity function for t:Type.Function.tchanges how t:Type.Tuple.t/1 works, by introducing minimum arity tuplesmakes inspecting types that could be confused as native terms less precarious by using parallelism with Type module macros.overhaul of how union merging works","ref":"changelog.html#0-0-5","title":"Changelog - 0.0.5","type":"extras"},{"doc":"make Type.fetch_type/3 use a sane call pattern","ref":"changelog.html#0-0-4","title":"Changelog - 0.0.4","type":"extras"},{"doc":"support for types of multiply-specced functionssupport for composed type module pipelinesdoc improvements contributed by @mbuhot and @kianmeng","ref":"changelog.html#0-0-3","title":"Changelog - 0.0.3","type":"extras"},{"doc":"some touchup:remove lambda examples from testsadd better support for aliased/composite builtinsbetter intro documentationsupport for fictional String.t/1 type; this will probably be pulled out into a plugin in the future.","ref":"changelog.html#0-0-2","title":"Changelog - 0.0.2","type":"extras"},{"doc":"initial push, with full functionality across most of elixir standard library","ref":"changelog.html#0-0-1","title":"Changelog - 0.0.1","type":"extras"}]